<?php
/**
 * @package     HSFirePHP!
 * @author      Hosting Skills A.s.b.l. - http://www.hosting-skills.lu
 * @copyright   Copyright (C) 2014 Hosting Skills A.s.b.l.. All rights reserved.
 * @license     GNU General Public License: http://www.gnu.org/copyleft/gpl.html
 */

// No direct access
defined( '_JEXEC' ) or die( 'Restricted access' );

/**
 * HSFirePHP! System Plugin
 */
class plgSystemHSFirePHP extends JPlugin {
    /**
     * Load the language file on instantiation. Note this is only available in Joomla 3.1 and higher.
     * If you want to support 3.0 series you must override the constructor
     *
     * @var    boolean
     * @since  3.1
     */
    protected $autoloadLanguage = true;

    /**
     * The validated parameters filled with default values if necessary.
     *
     * @var  array
     */
    protected $parameters = array();
    protected $options    = array();

    /**
     * Constructor.
     *
     * @param  object  $subject  The object to observe
     * @param  array   $config   An array that holds the plugin configuration
     * @since  1.0
     */
    public function __construct( &$subject, $config ) {
        parent::__construct( $subject, $config );

        $defaults = array();

        /* Activation */

        $defaults['sitesection'] = 3;
        $defaults['debugsystem'] = false;

        /* Authorization */

        $defaults['user'] = '';
        $defaults['groups'] = '';
        $defaults['accesslevels'] = '';

        /* Options */

        $defaults['maxobjectdepth'] = 5;
        $defaults['maxarraydepth'] = 5;
        $defaults['maxdepth'] = 10;
        $defaults['usenativejsonencode'] = true;
        $defaults['includelinenumbers'] = true;

        /* Handling */

        $defaults['errorhandling'] = true;
        $defaults['exceptionhandling'] = true;
        $defaults['assertionhandling'] = true;

        /* Demo */

        $defaults['demo'] = false;

        /* Parameters */

        $parameters['sitesection'] = $this->params->get('sitesection', $defaults['sitesection']);
        $parameters['debugsystem'] = $this->params->get('debugsystem', $defaults['debugsystem']);

        $parameters['user'] = $this->params->get('user', $defaults['user']);
        $parameters['groups'] = $this->params->get('groups', $defaults['groups']);
        $parameters['accesslevels'] = $this->params->get('accesslevels', $defaults['accesslevels']);

        $options['maxobjectdepth'] = $this->params->get('maxobjectdepth', $defaults['maxobjectdepth']);
        $options['maxarraydepth'] = $this->params->get('maxarraydepth', $defaults['maxarraydepth']);
        $options['maxdepth'] = $this->params->get('maxdepth', $defaults['maxdepth']);
        $options['usenativejsonencode'] = $this->params->get('usenativejsonencode', $defaults['usenativejsonencode']);
        $options['includelinenumbers'] = $this->params->get('includelinenumbers', $defaults['includelinenumbers']);

        $parameters['errorhandling'] = $this->params->get('errorhandling', $defaults['errorhandling']);
        $parameters['exceptionhandling'] = $this->params->get('exceptionhandling', $defaults['exceptionhandling']);
        $parameters['assertionhandling'] = $this->params->get('assertionhandling', $defaults['assertionhandling']);

        $parameters['demo'] = $this->params->get('demo', $defaults['demo']);

        $parameters = $this->validate($parameters, $defaults);
        $options    = $this->validate($options, $defaults);

        $this->parameters = array_merge($parameters, $options);
        $this->options    = $options;
    }

    /**
     * Do something onAfterInitialise
     */
    function onAfterInitialise() {
        if (class_exists('FirePHP') ||
            !file_exists(JPATH_SITE.'/plugins/system/hsfirephp/firephpcore/fb.php') ||
            !file_exists(JPATH_SITE.'/plugins/system/hsfirephp/firephpcore/FirePHP.class.php')) {
            return;
        }

        require_once JPATH_SITE.'/plugins/system/hsfirephp/firephpcore/fb.php';

        $firephp = FirePHP::getInstance(true);
        $firephp->setEnabled(false);

        if (!$this->canRun()) {
            return;
        }

        $firephp->setOptions($this->options);
        if ($this->parameters['errorhandling']) {
            $firephp->registerErrorHandler();
        }
        if ($this->parameters['exceptionhandling']) {
            $firephp->registerExceptionHandler();
        }
        if ($this->parameters['assertionhandling']) {
            assert_options(ASSERT_WARNING, false);
            $firephp->registerAssertionHandler();
        }
        $firephp->setEnabled(true);
    }

    /**
     * Do something onAfterRender
     */
    function onAfterRender() {
        if (!class_exists('FirePHP') || !$this->canRun() || !$this->parameters['demo']) {
            return;
        }

        $content = JResponse::getBody();
        $firephp = FirePHP::getInstance(true);

        preg_match_all('/HSFirePHP!::(LOG|ERROR|EXCEPTION|ASSERTION)/', $content, $matches, PREG_PATTERN_ORDER);
        if (in_array('LOG', $matches[1])) {
            $firephp->log('Plain Message');
            $firephp->info('Info Message');
            $firephp->warn('Warn Message');
            $firephp->error('Error Message');
            $firephp->log('Message','Optional Label');
        }
        if (in_array('ERROR', $matches[1]) && $this->parameters['errorhandling']) {
            // E_WARNING:
            trigger_error('', E_ERROR);
            // E_NOTICE:
            $E_NOTICE++;
            // E_USER_ERROR:
            trigger_error('E_USER_ERROR', E_USER_ERROR);
            // E_USER_WARNING:
            trigger_error('E_USER_WARNING', E_USER_WARNING);
            // E_USER_NOTICE:
            trigger_error('E_USER_NOTICE', E_USER_NOTICE);
            // E_STRICT:
            HSFirePHP::E_STRICT();
            // E_RECOVERABLE_ERROR:
            (string) new HSFirePHP();
            // E_DEPRECATED:
            split(':', '');
            // E_USER_DEPRECATED:
            trigger_error('E_USER_DEPRECATED', E_USER_DEPRECATED);
        }
        if (in_array('EXCEPTION', $matches[1]) && $this->parameters['exceptionhandling']) {
		    throw new Exception('Uncaught Exception');
        }
        if (in_array('ASSERTION', $matches[1]) && $this->parameters['assertionhandling']) {
            assert(is_bool(1));
        }
    }

    /**
     * Validate all $parameters and use the value from $defaults if validation fails.
     *
     * @param   array  $parameters  Retrieved values for the parameters of the plugin
     * @param   array  $defaults    Default values for the parameters of the plugin
     *
     * @return  array  Validated values for the parameters of the plugin
     */
    protected function validate($parameters, $defaults) {
        foreach ($parameters as $key => $value) {
            unset($parameters[$key]);
            $key = strtolower($key);
            $value = is_string($value) ? trim($value) : $value;
            $pattern = '';
            switch ($key) {
                case 'debugsystem':
                case 'usenativejsonencode':
                case 'includelinenumbers':
                case 'errorhandling':
                case 'exceptionhandling':
                case 'assertionhandling':
                case 'demo':
                    switch($value) {
                        case '1':
                        case 'true':
                        case  true :
                            $parameters[$key] = true;
                            break;

                        case '0':
                        case 'false':
                        case  false :
                            $parameters[$key] = false;
                            break;

                        default:
                            $parameters[$key] = $defaults[$key];
                            break;
                    }
                    break;

                case 'sitesection':
                    preg_match('/^[123]$/', $value, $matches);
                    if ($matches[0] == $value) {
                        $parameters[$key] = $value;
                    } else {
                        $parameters[$key] = $defaults[$key];
                    }
                    $parameters[$key] = (int) $parameters[$key];
                    break;

                case 'maxdepth':
                    $pattern = '\d|2';

                case 'maxobjectdepth':
                case 'maxarraydepth':
                    preg_match('/^1'.$pattern.'0|\d$/', $value, $matches);
                    if ($matches[0] == $value) {
                        $parameters[$key] = $value;
                    } else {
                        $parameters[$key] = $defaults[$key];
                    }
                    $parameters[$key] = (int) $parameters[$key];
                    break;

                default:
                    $parameters[$key] = $value;
                    break;
            }
        }

        return $parameters;
    }

    /**
     * Are the requirements fulfilled to run HSFirePHP!?
     *
     * @return  boolean  True if HSFirePHP! can run.
     */
    protected function canRun() {
        $app = JFactory::getApplication();
        if ($app->isAdmin() && 1 == $this->parameters['sitesection']) {
            return false;
        }
        if ($app->isSite()  && 2 == $this->parameters['sitesection']) {
            return false;
        }
        if ($this->parameters['debugsystem'] && !JDEBUG) {
            return false;
        }

        if ('' != $this->parameters['user'] || '' != $this->parameters['groups'] || '' != $this->parameters['accesslevels']) {
            $user = JFactory::getUser();
            $authorization = false;
            if ('' != $this->parameters['user'] && $this->parameters['user'] == $user->id) {
                $authorization = true;
            }
            if ('' != $this->parameters['groups']) {
                $intersection = array_intersect($user->getAuthorisedGroups(), $this->parameters['groups']);
                if (!empty($intersection)) {
                    $authorization = true;
                }
            }
            if ('' != $this->parameters['accesslevels']) {
                $intersection = array_intersect($user->getAuthorisedViewLevels(), $this->parameters['accesslevels']);
                if (!empty($intersection)) {
                    $authorization = true;
                }
            }
            if (!$authorization) {
                return false;
            }
        }

        return true;
    }
}

/**
 * Class used to generate an E_STRICT or an E_RECOVERABLE_ERROR error.
 */
class HSFirePHP {
    public function E_STRICT() {
    }
}